import tkinter as tk
from tkinter import ttk
from PIL import Image, ImageTk
import requests
from csv_reader import Csv_reader
import ast
import matplotlib.pyplot as plt
import numpy as np
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
import pandas as pd


class Pages(tk.Tk):
    def __init__(self):
        super().__init__()
        # set up board
        self.title("SECRET BASE!")
        self.geometry("1400x800")
        self.resizable(False, False)
        self.configure(bg="#f9cf95")
        self.df = Csv_reader().dataframe()

        # Create Title Label
        self.main_title = tk.Label(self, text="Secret Base!", bg="#f9584b", font=("", 60), width=40)
        self.main_title.pack()


        self.graph_title = tk.Label(self, text="Graph Space !", bg="#ff9c00", font=("", 30), width=80)
        self.graph_title.pack_forget()

        # create input Entry
        self.search_box = tk.Entry(self, width=40, font=("", 20))
        self.search_box.place(x=434, y=77)

        # create submit button
        self.btnSearch = tk.Button(self, text="Search", font=("", 20), command=self.search)
        self.btnSearch.place(x=963, y=77)

        # Create Light switch button
        self.light_switch = tk.Button(self, text="Light Mode", font=("", 20), command=self.light_switch)
        self.light_switch.place(x=70, y=77)

        # Create Graph spectate button
        self.btnGraph = tk.Button(self, text="Graph", font=("", 20), command=self.graph)
        self.btnGraph.place(x=1309, y=77)

        # Create Graph spectate button
        self.btnGoBack = tk.Button(self, text="Back", font=("", 20), command=self.go_back)
        self.btnGoBack.pack_forget()

        # create quit button
        self.btn_quit = tk.Button(self, text="Quit", font=("", 20), command=self.destroy)
        self.btn_quit.place(x=0, y=77)

        # create picture label to spare the picture space
        self.pic_label = tk.Label(self, text="picture", font=("", 20), width=30, height=20)
        self.pic_label.pack_forget()

        # Create info box to spare the info space
        self.info_label = tk.Label(self, text="info", font=("", 20), width=30, height=20)
        self.info_label.pack_forget()

        # Create Listbox for graph that User want to spectate
        self.graph_type = tk.Listbox(self)
        self.graph_type.pack_forget()

        # Create Option menu for select the graph
        self.start = ["Select Graph"]
        self.options = ["Distribution", "Everyday"]
        self.selected_option = tk.StringVar(self)
        self.selected_option.set(self.start[0])
        self.option_menu = tk.OptionMenu(self, self.selected_option, *self.options, command=self.graph_selection)
        self.option_menu.pack_forget()

        self.histogram_graph = None
        self.everyday_graph = None


    def search(self):
        name = self.search_box.get()

        # delete previous info_list if it exists
        for widget in self.winfo_children():
            if isinstance(widget, ttk.Treeview):
                widget.destroy()

        # delete show_na if it exists
        for widget in self.winfo_children():
            if isinstance(widget, tk.Label) and widget.cget("text") == "No Anime Available.":
                widget.destroy()


        info_list = ttk.Treeview()
        showing_info = ["title", "type", "score", "genres", "studios"]
        info_list["columns"] = showing_info
        info_list.column("#0", anchor="center", width=0, stretch=False)
        info_list.column("title", anchor="center", width=300)
        info_list.heading("title", text="title", anchor="center")
        info_list.column("type", anchor="center", width=50)
        info_list.heading("type", text="type", anchor="center")
        info_list.column("score", anchor="center", width=80)
        info_list.heading("score", text="score", anchor="center")
        info_list.column("genres", anchor="center", width=200)
        info_list.heading("genres", text="genres", anchor="center")
        info_list.column("studios", anchor="center", width=200)
        info_list.heading("studios", text="studios", anchor="center")
        info_list.configure(height=min(len(self.df), 5))

        def on_select(event):
            for widget in self.winfo_children():
                if isinstance(widget, tk.Label) and widget == self.pic_label or isinstance(widget, tk.Label) \
                        and widget.cget("text") == "Connect to the internet for pictures." or \
                        isinstance(widget, tk.Label) and widget.cget("text") == "No picture to show." or\
                        isinstance(widget, tk.Label) and widget == self.info_label:
                    widget.destroy()
            selected_item = event.widget.selection()[0]
            selected_title = info_list.item(selected_item)["values"][0]
            self.show_picture(selected_title)
            self.show_info(selected_title)
        info_list.bind("<<TreeviewSelect>>", on_select)
        found = False
        limit = 0
        for i in range(len(self.df)):
            # Search by Title
            if name.lower() in str(self.df["title"][i]).lower() \
                    or name.lower() in str(self.df["title_japanese"][i]) \
                    or name.lower() in str(self.df["title_english"][i]).lower() \
                    or name.lower() in [ast.literal_eval(self.df["title_synonyms"][i])[l] for l in
                                range(len(ast.literal_eval(self.df["title_synonyms"][i])))]:
                if limit < 100:
                    info_list.insert(parent="", index=i, text='', values=(self.df["title"][i], self.df["type"][i], self.df["score"][i],
                                                                          self.df["genres"][i], self.df["studios"][i]))
                limit += 1
                found = True
            # Search by Genres
            if name.lower() in [ast.literal_eval(self.df["genres"][i])[l].lower() for l in
                                range(len(ast.literal_eval(self.df["genres"][i])))]:
                if limit < 100:
                    info_list.insert(parent="", index=i, text='', values=(self.df["title"][i], self.df["type"][i], self.df["score"][i],
                                                                          self.df["genres"][i], self.df["studios"][i]))
                limit += 1
                found = True
            # Search by Studios
            if name.lower() in [ast.literal_eval(self.df["studios"][i])[l].lower() for l in
                                range(len(ast.literal_eval(self.df["studios"][i])))]:
                if limit < 100:
                    info_list.insert(parent="", index=i, text='', values=(self.df["title"][i], self.df["type"][i], self.df["score"][i],
                                                                          self.df["genres"][i], self.df["studios"][i]))
                limit += 1
                found = True


        if not found:
            show_na = tk.Label(self, text="No Anime Available.", bg="black", font=("", 20), width=40)
            show_na.place(x=436, y=112)
        else:
            info_list.place(x=331, y=110)

    def show_picture(self, title):
        for i in range(len(self.df["title"])):
            if str(title) == self.df["title"][i]:
                try:
                    response = requests.get(self.df["main_picture"][i], stream=True)
                    response.raw.decode_content = True
                    image = Image.open(response.raw)

                    # calculate new width based on maximum height and original aspect ratio
                    max_height = 400
                    width, height = image.size
                    aspect_ratio = width / height
                    new_width = int(aspect_ratio * max_height)

                    # create new image with new width and maximum height
                    resized_image = image.resize((new_width, max_height))
                    photo_image = ImageTk.PhotoImage(resized_image)
                    self.pic_label = tk.Label(image=photo_image)
                    self.pic_label.image = photo_image
                    self.pic_label.place(x=25, y=355)

                except requests.exceptions.ConnectionError:
                    for widget in self.winfo_children():
                        if isinstance(widget, tk.Label) and widget.cget("text") == \
                                "Connect to the internet for pictures.":
                            widget.destroy()
                    error_label = tk.Label(text="Connect to the internet for pictures.", font=("", 20), width=25, height=15)
                    error_label.place(x=50, y=375)

                except requests.exceptions.MissingSchema:
                    for widget in self.winfo_children():
                        if isinstance(widget, tk.Label) and widget.cget("text") == \
                                "No picture to show.":
                            widget.destroy()
                    error_label = tk.Label(text="No picture to show.", font=("", 20), width=35, height=30)
                    error_label.place(x=25, y=355)

    def show_info(self, title):
        for i in range(len(self.df["title"])):
            if str(title) == self.df["title"][i]:
                text = f"""Title : {self.df["title"][i]}\nType : {self.df["type"][i]}\nScore : {self.df["score"][i]}\n\
Airing Status : {self.df["status"][i]}\nNumber of Episodes : {self.df["episodes"][i]}\n\
Airing Year : {self.df["start_year"][i]}\nGenres : {self.df["genres"][i]}\nThemes : {self.df["themes"][i]}\n\
Studios : {self.df["studios"][i]}\nSynopsis : {self.df["synopsis"][i]}"""
                self.info_label = tk.Label(text=text, font=("", 10), width=130, wraplength=900,
                                           justify="left", height=30)
                self.info_label.place(x=380, y=375)


    def light_switch(self):
        current_bg = self.cget("bg")
        color = ['black', '#f9cf95']
        if current_bg == color[0]:
            new_bg = color[1]
            self.light_switch.configure(text="Dark Mode")
        else:
            new_bg = color[0]
            self.light_switch.configure(text="Light Mode")
        self.configure(bg=new_bg)

    def graph(self):
        for widget in self.winfo_children():
            if widget != self.main_title:
                widget.pack_forget()
                widget.place_forget()
                self.btnGoBack.pack()
                self.graph_title.pack(before=self.btnGoBack)
                self.option_menu.pack()


    def go_back(self):
        self.search_box.place(x=434, y=77)
        self.btnSearch.place(x=963, y=77)
        self.light_switch.place(x=70, y=77)
        self.btnGraph.place(x=1309, y=77)
        self.btn_quit.place(x=0, y=77)
        self.btnGoBack.pack_forget()
        self.graph_title.pack_forget()
        self.option_menu.pack_forget()
        if self.everyday_graph is not None:
            self.everyday_graph.get_tk_widget().destroy()
        if self.histogram_graph is not None:
            self.histogram_graph.get_tk_widget().destroy()

    def graph_selection(self, selection):
        if selection == "Distribution":
            self.show_distribution_graph()
        elif selection == "Everyday":
            self.show_everyday_graph()
        else:
            # Do nothing if the user selects the default option
            pass

    def show_distribution_graph(self):
        for widget in self.winfo_children():
            if widget != self.main_title or widget != self.graph_title or widget != self.btnGoBack:
                widget.place_forget()
        if self.everyday_graph is not None:
            self.everyday_graph.get_tk_widget().destroy()
        if self.histogram_graph is not None:
            self.histogram_graph.get_tk_widget().destroy()
        # Code to show the distribution graph goes here
        new_df = self.df["score"].dropna()

        histogram = plt.figure(figsize=(10, 5))

        # Create some random movie scores
        movie_scores = new_df

        # Plot the histogram
        plt.hist(movie_scores, bins=20, range=(1, 11))

        # Set the x-axis and y-axis labels
        plt.title('Histogram of Score')
        plt.xlabel('Anime Score')
        plt.ylabel('Frequency')

        # Set the x-axis tick marks
        plt.xticks(np.arange(1, 10.5, step=0.5))

        # Set the x-axis limits
        plt.xlim(1, 10)

        self.histogram_graph = FigureCanvasTkAgg(histogram, master=self)
        self.histogram_graph.draw()
        self.histogram_graph.get_tk_widget().pack()

    def show_everyday_graph(self):
        for widget in self.winfo_children():
            if widget != self.main_title or widget != self.graph_title or widget != self.btnGoBack:
                widget.place_forget()
        if self.everyday_graph is not None:
            self.everyday_graph.get_tk_widget().destroy()
        if self.histogram_graph is not None:
            self.histogram_graph.get_tk_widget().destroy()
        everyday_df = self.df.copy()
        everyday_df['score'].dropna()
        # Convert the 'airing year' column to datetime format
        everyday_df['airing year'] = pd.to_datetime(everyday_df['start_year'], format='%Y')

        # Group the data by year and calculate the average score for each year
        grouped_data = everyday_df.groupby(everyday_df['airing year'].dt.year)['score'].mean()

        # Interpolate the missing values
        grouped_data = grouped_data.interpolate()

        everyday_graph = plt.figure(figsize=(10, 5))

        # Plot the line graph
        plt.plot(grouped_data.index, grouped_data.values)

        # Set the x-axis tick marks to show only every 10 years
        plt.xticks(np.arange(1920, 2021 + 1, 10))

        # Set the x-axis and y-axis labels
        plt.xlabel('Year')
        plt.ylabel('Average Score')

        # Set the title of the graph
        plt.title('Average Anime Score by Year')
        self.everyday_graph = FigureCanvasTkAgg(everyday_graph, master=self)
        self.everyday_graph.draw()
        self.everyday_graph.get_tk_widget().pack()
